const {entrypoints, storage, shell} = require("uxp");
const {app, ExportLayerOptions, BitmapCompression, PdfDisplayTitleOptions} = require("indesign");
const fs = storage.localFileSystem;

const apiBaseUrl = "https://simplebooklet.com"; // Change this to your API base URL

console.log("Using API Base URL:", apiBaseUrl);


let openUrl = null;

entrypoints.setup({
  panels: {
    showPanel: {
      show({node} = {}) {
      }
    }
  }
});

showAlert = () => {
  const dialog = app.dialogs.add();
  const col = dialog.dialogColumns.add();
  const colText = col.staticTexts.add();
  colText.staticLabel = "Congratulations! You just executed your first command.";

  dialog.canCancel = false;
  dialog.show();
  dialog.destroy();
}

async function exportPDF() {
  try {
    // Get all PDF Export Presets
    const pdfExportPresets = app.pdfExportPresets.everyItem().name;
    const document = app.activeDocument;
    const documentTitle = document.name;
    const documentDescription = document.metadataPreferences.description;
    const documentId = getDocumentId();
    console.log("document name: ", documentTitle);
    console.log("PDF Export Presets:", pdfExportPresets);
    console.log("Document ID:", documentId);

    // document.metadataPreferences.documentTitle = documentTitle;
    // Set the document title in metadata if it isn't already set, and remove .indd extension if it has that at the end
    if (!document.metadataPreferences.documentTitle) {
      document.metadataPreferences.documentTitle = documentTitle.replace(/\.indd$/, '');
    }
    document.metadataPreferences.description = `indesign_id:${documentId}`;

    // Try using custom preset to include hyperlinks and bookmarks and smaller file size
    // add if it doesn't exist
    if (!pdfExportPresets.includes('Simplebooklet Export')) {
      await app.pdfExportPresets.add({
        name: 'Simplebooklet Export',
      });
    }
    //
    // create PDF export preset
    const presetName = "Simplebooklet Export"; // Change this to whichever PDF Export preset you want to use
    const pdfExportPreset = app.pdfExportPresets.itemByName(presetName);
    // Update the properties of the existing preset
    pdfExportPreset.includeHyperlinks = true;
    pdfExportPreset.exportBookmarks = true;
    pdfExportPreset.optimizeForWeb = true;
    pdfExportPreset.compression = BitmapCompression.JPEG;
    pdfExportPreset.exportWhichLayers = ExportLayerOptions.EXPORT_VISIBLE_LAYERS;
    pdfExportPreset.pdfDisplayTitle = PdfDisplayTitleOptions.DISPLAY_DOCUMENT_TITLE;

    // generate a new File object to save the PDF as '/tmp/exported.pdf'
    const tmpFolder = await fs.getTemporaryFolder();
    const tmpFile = await tmpFolder.createFile('exported.pdf', {overwrite: true});

    await document.exportFile('Adobe PDF (Print)', tmpFile, false, pdfExportPreset);
    console.log("tmpFile:", tmpFile);

    document.metadataPreferences.description = documentDescription; // Reset the description to the original value

    return tmpFile; // This will return the exported PDF file as a Blob or file object
  } catch (error) {
    console.log("Failed to export PDF:", error);
    showErrorMessage(error.message);
  }
}

async function uploadPDFToSimplebooklet(pdfFile) {
  console.log("Uploading PDF to Simplebooklet...", pdfFile);

  // get pdf file contents
  const pdfData = await pdfFile.read({format: storage.formats.binary});
  const documentId = getDocumentId();
  console.log("PDF data:", pdfData.byteLength);

  const formData = new FormData();
  formData.append('pdf', pdfData, 'file.pdf'); // The 'pdf' field contains the PDF file
  formData.append('indesign_id', documentId); // Additional field for InDesign document ID

  const response = await fetch(`${apiBaseUrl}/adobe/indesign_app/pdf_export`, {
    method: 'POST',
    body: formData
  });

  if (!response.ok) {
    console.log("Failed to upload PDF:", response.statusText);
    showErrorMessage("Failed to upload PDF: " + response.statusText);
  } else {
    const result = await response.json();
    console.log("PDF uploaded successfully:", result);
    // createLaunchButton(result.data.url);
    openUrl = result.data.url;
    // showContainer('open-container');
    // for debug test replace local with local2 in the url
    // openUrl = openUrl.replace('local.', 'local2.');
    try {
      await shell.openExternal(openUrl);
      console.log("Success to open URL:", openUrl);
      showContainer('success-container');
    } catch (error) {
      console.log("Failed to open URL:", error);
      showContainer('export-container');
    }

  }

  // TODO: Use the following code to upload PDF using a stream instead of FormData... keeping this for now in case we need to revisit but for the moment, FormData works fine
  // // Assuming you already have the tmpFile from previous steps
  // const pdfStream = await pdfFile.createReadStream();
  // console.log("PDF stream:", pdfStream);
  //
  // const formData = new FormData();
  // formData.append("file", pdfStream, "exported.pdf"); // You can provide the file name here
  //
  // console.log("Uploading PDF to Simplebooklet...", formData);
  // const response = await fetch('https://local.simplebooklet.com/adobe/indesign_app/pdf_export', {
  //   method: 'POST',
  //   body: formData,
  //   headers: {
  //     // 'Authorization': 'Bearer YOUR_API_KEY',
  //   }
  // });
  // console.log("Response:", response);
  //
  // if (!response.ok) {
  //   console.error("Failed to upload PDF:", response.statusText);
  // } else {
  //   const result = await response.json();
  //   console.log("PDF uploaded successfully:", result);
  //   createLaunchButton(result.data.url);
  // }
}

function showContainer(containerId) {
  // hide all div having a container class
  document.querySelectorAll('.container').forEach((element) => {
    element.style.display = 'none';
  });
  document.getElementById(containerId).style.display = 'block';
}

function getDocumentId() {
  let documentId = app.activeDocument.metadataPreferences.getProperty("http://ns.adobe.com/xap/1.0/mm/", 'DocumentID');
  if (documentId) {
    documentId = documentId.replace('xmp.did:', '');
  }
  return documentId;
}

function showErrorMessage(message) {
  document.getElementById('error-message').innerText = message;
  showContainer('error-container');
}

document.getElementById('export-pdf').addEventListener('click', async () => {
  showContainer('exporting-container');
  const pdfFile = await exportPDF();
  console.log("PDF file:", pdfFile);
  if (pdfFile) {
    await uploadPDFToSimplebooklet(pdfFile);
  }
  // } else {
  //   showContainer('export-container')
  // }
});

document.getElementById('open-simplebooklet').addEventListener("click", () => {
  shell.openExternal(apiBaseUrl+'/dashboard.php');
});

// document.getElementById('export-again').addEventListener('click', async () => {
//   showContainer('export-container');
// });
document.querySelectorAll('.export-again').forEach(button => {
  button.addEventListener('click', async () => {
    showContainer('export-container');
  });
});
